/*
 * Copyright 2010-2019 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package org.jetbrains.kotlin.nj2k.inference.common

import org.jetbrains.kotlin.idea.resolve.ResolutionFacade
import org.jetbrains.kotlin.nj2k.inference.AbstractConstraintCollectorTest
import org.jetbrains.kotlin.nj2k.inference.common.collectors.CallExpressionConstraintCollector
import org.jetbrains.kotlin.nj2k.inference.common.collectors.CommonConstraintsCollector
import org.jetbrains.kotlin.nj2k.inference.common.collectors.FunctionConstraintsCollector
import org.jetbrains.kotlin.nj2k.inference.nullability.NullabilityConstraintBoundProvider
import org.jetbrains.kotlin.psi.KtTypeElement

abstract class AbstractCommonConstraintCollectorTest : AbstractConstraintCollectorTest() {
    override fun createInferenceFacade(resolutionFacade: ResolutionFacade): InferenceFacade =
        InferenceFacade(
            object : ContextCollector(resolutionFacade) {
                override fun ClassReference.getState(typeElement: KtTypeElement?): State =
                    State.UNKNOWN
            },
            ConstraintsCollectorAggregator(
                resolutionFacade,
                NullabilityConstraintBoundProvider(),
                listOf(
                    CommonConstraintsCollector(),
                    CallExpressionConstraintCollector(),
                    FunctionConstraintsCollector(ResolveSuperFunctionsProvider(resolutionFacade))
                )
            ),
            BoundTypeCalculatorImpl(resolutionFacade, BoundTypeEnhancer.ID),
            object : StateUpdater() {
                override fun TypeElementBasedTypeVariable.updateState() = Unit
            },
            object : DefaultStateProvider() {
                override fun defaultStateFor(typeVariable: TypeVariable): State = State.LOWER
            },
            renderDebugTypes = true
        )
}